@@ -1,6 +1,6 @@
 /* startprogop.h
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2021 Ralf Hoffmann.
+ * Copyright (C) 2001-2023 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -32,6 +32,7 @@ class AWindow;
 class PersistentStringList;
 class ChooseButton;
 class Button;
+class ListerMode;
 
 class StartProgOp:public FunctionProto
 {
@@ -55,13 +56,18 @@ public:
                 STARTPROGOP_START_IN_TERMINAL,
                 STARTPROGOP_START_IN_TERMINAL_AND_WAIT4KEY,
                 STARTPROGOP_SHOW_OUTPUT,
-                STARTPROGOP_SHOW_OUTPUT_INT } startprogstart_t;
+                STARTPROGOP_SHOW_OUTPUT_INT,
+                STARTPROGOP_SHOW_OUTPUT_OTHER_SIDE,
+                STARTPROGOP_SHOW_OUTPUT_CUSTOM_ATTRIBUTE } startprogstart_t;
   void setStart(startprogstart_t);
   void setGlobal(bool);
   void setRequestFlags(bool);
   void setInBackground( bool );
   void setViewStr( std::string );
   void setDontCD( bool );
+    void setFollowActive( bool );
+    void setWatchFile( bool );
+    void setSeparateEachEntry( bool nv );
   void setGUIMsg( std::string msg );
 protected:
   static const char *name;
@@ -73,6 +79,9 @@ protected:
   bool global;
   bool inbackground;
   bool dontcd;
+    bool m_follow_active = false;
+    bool m_watch_file = false;
+    bool m_separate_each_entry = false;
   
   // temp variables
   Lister *startlister,*endlister;
@@ -82,19 +91,28 @@ protected:
   class SettingsWidgets
   {
   public:
-    SettingsWidgets( class StringGadget *sg,
-                     class CycleButton *cyb,
-                     ChooseButton *gcb,
-                     ChooseButton *ibcb,
-                     ChooseButton *dcdcb );
+      SettingsWidgets( class AContainer *sg_ac,
+                       class StringGadget *sg,
+                       class CycleButton *cyb,
+                       ChooseButton *gcb,
+                       ChooseButton *ibcb,
+                       ChooseButton *dcdcb,
+                       ChooseButton *follow_active_cb,
+                       ChooseButton *watch_file_cb,
+                       ChooseButton *separate_each_entry_cb );
     SettingsWidgets( const SettingsWidgets &other );
     SettingsWidgets &operator=( const SettingsWidgets &other );
     int validate();
+      void updateVisibility();
+      class AContainer *m_sg_ac;
     class StringGadget *_sg;
-    class CycleButton *_cyb;
+    class CycleButton *m_cyb;
     ChooseButton *_gcb;
     ChooseButton *_ibcb;
     ChooseButton *_dcdcb;
+      ChooseButton *m_follow_active_cb;
+      ChooseButton *m_watch_file_cb;
+      ChooseButton *m_separate_each_entry_cb;
   };
     std::unique_ptr< SettingsWidgets > buildW( AWindow *win );
 
@@ -103,13 +121,13 @@ protected:
   public:
     FiletypeWindow( class FieldListView *lv,
                     StringGadget *sg,
-                    FlatTypeList *flatlist );
+                    std::unique_ptr< FlatTypeList > flatlist );
     ~FiletypeWindow();
     FiletypeWindow( const FiletypeWindow &other );
     FiletypeWindow &operator=( const FiletypeWindow &other );
     class FieldListView *_lv;
     StringGadget *m_filter_sg;
-    FlatTypeList *_flatlist;
+    std::unique_ptr< FlatTypeList > _flatlist;
 
       void updateView();
   };
@@ -131,8 +149,8 @@ protected:
   {
   public:
     StartSettings();
-    StartSettings( const StartSettings &other );
-    StartSettings &operator=( const StartSettings &other );
+    StartSettings( const StartSettings &other ) = default;
+      StartSettings &operator=( const StartSettings &other ) = default;
     enum StartModes { CANCEL, STARTPROG, HANDLE_TYPE, HANDLE_ARCHIVE } _startmode;
     std::string _command;
     class WCFiletype *_type;
@@ -143,6 +161,9 @@ protected:
     bool _global;
     bool _inbackground;
     bool _dontcd;
+      bool m_follow_active = false;
+      bool m_watch_file = false;
+      bool m_separate_each_entry = false;
   };
   std::unique_ptr< StartSettings > showGUI();
 
@@ -152,12 +173,22 @@ protected:
     void updateSGFromComplete( StringGadget *sg,
                                PersistentStringList &history,
                                std::shared_ptr< AJSON::JSONType > &filetype_history,
-                               FieldListView *lv );
+                               FieldListView *lv,
+                               SettingsWidgets &sw );
     void updateSGFromRow( int row,
                           PersistentStringList &history,
-                          std::shared_ptr< AJSON::JSONType > &filetype_history );
+                          std::shared_ptr< AJSON::JSONType > &filetype_history,
+                          SettingsWidgets &sw );
     void updateFiletypeHistory( std::shared_ptr< AJSON::JSONType > &filetype_history,
-                                const std::string &base );
+                                const std::string &base,
+                                const StartSettings &settings );
+    void set_history_json( std::shared_ptr< AJSON::JSONObject > &jobj,
+                           const std::string &base,
+                           const StartSettings &settings );
+    int parse_history_json( std::shared_ptr< AJSON::JSONObject > &jobj,
+                            StartSettings &settings );
+    void apply_settings( const StartSettings &settings,
+                         SettingsWidgets &sw );
 
   std::string default_file;
   std::string m_default_file_filetype;
@@ -172,7 +203,7 @@ protected:
                           ANY_COMPLETION
     };
     
-    std::list< std::pair< completion_type, std::string > > m_current_completions;
+    std::list< std::pair< completion_type, StartSettings > > m_current_completions;
     std::string m_current_original_file;
     bool m_completion_enabled;
     std::string m_previous_sg_content;
